@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: module.h 559 2008-06-13 11:32:55Z garden $
+ *   $Id: module.h 606 2008-08-27 06:37:41Z rd235 $
  *
  */
 #ifndef _UM_VIEW_MODULE_H
@@ -101,6 +101,7 @@ struct ioctl_len_req {
 struct binfmt_req {
 	char *path;
 	char *interp;
+	char *extraarg;
 	int flags;
 };
 
@@ -143,7 +144,7 @@ struct service {
 	 * the life of a process, can be used as an index for internal data
 	 * pumpid is the similar id for the parent process, -1 if it does not exist
 	 *
-	 * MC_PROC | MC_REM: int id
+	 * MC_PROC | MC_REM: int umpid
 	 * is the garbage collection function for the data that addproc may have created
 	 *
 	 * MC_MOUNT | MC_ADD:
