@@ -17,7 +17,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umbinfmt.c 282 2007-01-18 12:33:03Z garden $
+ *   $Id: umbinfmt.c 648 2009-04-22 22:50:01Z rd235 $
  *
  */   
 #include <stdio.h>
@@ -34,7 +34,6 @@
 #include <dlfcn.h>
 #include <pthread.h>
 #include <linux/types.h>
-#include <linux/dirent.h>
 #include <linux/unistd.h>
 #include <stdarg.h>
 #include <stdint.h>
@@ -153,30 +152,6 @@ static void printdebug(int level, const char *file, const int line, const char *
 }
 #endif
 
-static void cutdots(char *path)
-{
-	int l=strlen(path);
-	l--;
-	if (path[l]=='.') {
-		l--;
-		if(path[l]=='/') {
-			if (l!=0) path[l]=0; else path[l+1]=0;
-		} else if (path[l]=='.') {
-			l--;
-			if(path[l]=='/') {
-				while(l>0) {
-					l--;
-					if (path[l]=='/')
-						break;
-				}
-				if(path[l]=='/') {
-					if (l!=0) path[l]=0; else path[l+1]=0;
-				}
-			}
-		}
-	}
-}
-
 /* search a binfmt, returns the context i.e. the index of info for mounted file
  * -1 otherwise */
 static struct umbinfmt *searchbmfile(char *path)
@@ -186,7 +161,6 @@ static struct umbinfmt *searchbmfile(char *path)
 	epoch_t maxepoch=0;
 	int maxi=-1;
 	PRINTDEBUG(0,"SearchContext:%s\n",path);
-	cutdots(path);
 	for (i=0;i<mnttabmax;i++)
 	{
 		epoch_t e;
@@ -980,6 +954,8 @@ static int common_stat64(struct umbinfmt *fc,struct umregister *reg, struct stat
 }
 */
 
+#if 0
+/* 09 2008 f-syscall unification */
 static long umbinfmt_fstat64(int fd, struct stat64 *buf64)
 {
 	if (fd < 0 || filetab[fd] == NULL) {
@@ -989,6 +965,7 @@ static long umbinfmt_fstat64(int fd, struct stat64 *buf64)
 		return common_stat64(filetab[fd]->bfmount, filetab[fd]->reg,buf64);
 	}
 }
+#endif
 
 /*
 static long umbinfmt_fstat64(int fd, struct stat64 *buf64)
@@ -1182,11 +1159,11 @@ init (void)
 #if !defined(__x86_64__)
 	SERVICESYSCALL(s, stat64, umbinfmt_stat64);
 	SERVICESYSCALL(s, lstat64, umbinfmt_lstat64);
-	SERVICESYSCALL(s, fstat64, umbinfmt_fstat64);
+	//SERVICESYSCALL(s, fstat64, umbinfmt_fstat64);
 #else
 	SERVICESYSCALL(s, stat, umbinfmt_stat64);
 	SERVICESYSCALL(s, lstat, umbinfmt_lstat64);
-	SERVICESYSCALL(s, fstat, umbinfmt_fstat64);
+	//SERVICESYSCALL(s, fstat, umbinfmt_fstat64);
 #endif
 	SERVICESYSCALL(s, access, umbinfmt_access);
 	SERVICESYSCALL(s, lseek, umbinfmt_lseek);
