@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umdev.c 470 2008-04-26 19:53:15Z rd235 $
+ *   $Id: umdev.c 648 2009-04-22 22:50:01Z rd235 $
  *
  */   
 #include <stdio.h>
@@ -35,7 +35,6 @@
 #include <dlfcn.h>
 #include <pthread.h>
 #include <linux/types.h>
-#include <linux/dirent.h>
 #include <linux/unistd.h>
 #include <stdarg.h>
 #include <stdint.h>
@@ -117,30 +116,6 @@ static void printdebug(int level, const char *file, const int line, const char *
 }
 #endif
 
-static void cutdots(char *path)
-{
-	int l=strlen(path);
-	l--;
-	if (path[l]=='.') {
-		l--;
-		if(path[l]=='/') {
-			if (l!=0) path[l]=0; else path[l+1]=0;
-		} else if (path[l]=='.') {
-			l--;
-			if(path[l]=='/') {
-				while(l>0) {
-					l--;
-					if (path[l]=='/')
-						break;
-				}
-				if(path[l]=='/') {
-					if (l!=0) path[l]=0; else path[l+1]=0;
-				}
-			}
-		}
-	}
-}
-
 static int search_plusnum(const char *path, const char *base, int nsubdev)
 {
 	int len=strlen(base);
@@ -175,7 +150,6 @@ static struct umdev *searchdevice(char *path)
 	int maxi=-1;
 
 	PRINTDEBUG(0,"SearchContext:%s\n",path);
-	cutdots(path);
 	for (i=0;i<devicetabmax;i++)
 	{
 		epoch_t e;
@@ -200,6 +174,7 @@ static struct umdev *searchdevice(char *path)
 				/* set epoch */
 				epoch_t prevepoch=um_setepoch(devicetab[i]->tst.epoch);
 				if (((e=tst_matchingepoch(&(devicetab[i]->tst))) > maxepoch) &&  /* Epoch compatible  AND */
+						/* RD 08.2008, why can't we use um_mod_getpathstat? try it */
 						(stat64(path,&buf) == 0) &&  /* stat okay AND */
 						((((devicetab[i]->mode & S_IFMT) == 0) ||  /* the same kind of special file (or 0, any kind)  AND */
 							((buf.st_mode & S_IFMT) ==  (devicetab[i]->mode & S_IFMT))) &&
@@ -837,6 +812,8 @@ static int common_stat64(struct umdev *fc, char type, dev_t device, struct stat6
 }
 */
 
+#if 0
+/* 09 2008 f-syscall unification */
 static long umdev_fstat64(int fd, struct stat64 *buf64)
 {
 	if (fd < 0 || filetab[fd] == NULL) {
@@ -865,6 +842,7 @@ static long umdev_fstat64(int fd, struct stat64 *buf64)
 			return common_stat64(fc,filetab[fd]->type,filetab[fd]->device,buf64);
 	}
 }
+#endif
 
 /*
 static long umdev_fstat64(int fd, struct stat64 *buf64)
@@ -1254,7 +1232,7 @@ init (void)
 #if !defined(__x86_64__)
 	SERVICESYSCALL(s, stat64, umdev_stat64);
 	SERVICESYSCALL(s, lstat64, umdev_lstat64);
-	SERVICESYSCALL(s, fstat64, umdev_fstat64);
+	//SERVICESYSCALL(s, fstat64, umdev_fstat64);
 #endif
 	SERVICESYSCALL(s, access, umdev_access);
 	SERVICESYSCALL(s, lseek, umdev_lseek);
@@ -1263,7 +1241,7 @@ init (void)
 #endif
 	//SERVICESYSCALL(s, mknod, umdev_mknod);
 	SERVICESYSCALL(s, chown, umdev_chown);
-	SERVICESYSCALL(s, fchown, fchown);
+	//SERVICESYSCALL(s, fchown, fchown);
 	SERVICESYSCALL(s, chmod, umdev_chmod);
 	//SERVICESYSCALL(s, fchmod, fchmod);
 	SERVICESYSCALL(s, fsync, umdev_fsync); 
