@@ -20,7 +20,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umfuse.c 465 2008-04-19 06:21:14Z rd235 $
+ *   $Id: umfuse.c 666 2009-05-25 07:52:30Z rd235 $
  *
  */   
 #include <unistd.h>
@@ -37,7 +37,6 @@
 #include <pthread.h>
 #include <fuse.h>
 #include <linux/types.h>
-#include <linux/dirent.h>
 #include <linux/unistd.h>
 #include "module.h"
 #include "libummod.h"
@@ -86,6 +85,7 @@ static struct service s;
 struct fuse {
 	char *filesystemtype;
 	char *path;
+	char **exceptions;
 	short pathlen;
 	void *dlhandle;
 	struct timestamp tst;
@@ -196,30 +196,47 @@ static void printdebug(int level, const char *file, const int line, const char *
 #endif
 #endif
 
-static cutdots(char *path)
+/* search for exceptions returns 1 if it is an exception */
+
+static inline int isexception(char *path, char **exceptions, struct fuse_context *fc)
 {
-	int l=strlen(path);
-	l--;
-	if (path[l]=='.') {
-		l--;
-		if(path[l]=='/') {
-			if (l!=0) path[l]=0; else path[l+1]=0;
-		} else if (path[l]=='.') {
-			l--;
-			if(path[l]=='/') {
-				while(l>0) {
-					l--;
-					if (path[l]=='/')
-						break;
-				}
-				if(path[l]=='/') {
-					if (l!=0) path[l]=0; else path[l+1]=0;
-				}
+	if (__builtin_expect((exceptions == NULL && !(fc->fuse->flags & FUSE_MERGE)),1))
+		return 0;
+	else {
+		if (exceptions) {
+			while (*exceptions != 0) {
+				int len=strlen(*exceptions);
+				if (strncmp(path,*exceptions,len) == 0 &&
+						(path[len] == '/' || path[len]=='\0'))
+					return 1;
+				exceptions ++;
 			}
 		}
+		if (fc->fuse->flags & FUSE_MERGE) {
+			struct stat buf;
+			struct fuse_context *oldfc=fuse_gs_context(fc);
+			int rv=fc->fuse->fops.getattr(path,&buf);
+			fuse_set_context(oldfc);
+			return (rv < 0);
+		}
+		return 0;
 	}
 }
 
+static inline void freeexceptions(char **exceptions)
+{
+	if (__builtin_expect((exceptions == NULL),1))
+		 return;
+	 else {
+		 char **excscan=exceptions;
+		 while (*excscan != 0) {
+			 free(*excscan);
+			 excscan++;
+		 }
+		 free(exceptions);
+	 }
+}
+
 /* search a path, returns the context i.e. the index of info for mounted file
  * -1 otherwise */
 static struct fuse_context *searchcontext(char *path,int exact)
@@ -229,7 +246,6 @@ static struct fuse_context *searchcontext(char *path,int exact)
 	epoch_t maxepoch=0;
 	int maxi=-1;
 	GDEBUG(1,"SearchContext:%s-%s ENTER!",path, exact?"EXACT":"SUBSTR");
-	cutdots(path);
 	for (i=0;i<fusetabmax;i++)
 	{
 		epoch_t e;
@@ -242,7 +258,7 @@ static struct fuse_context *searchcontext(char *path,int exact)
 				}
 			} else {
 				int len=fusetab[i]->fuse->pathlen;
-				if ((strncmp(path,fusetab[i]->fuse->path,len) == 0 && (path[len] == '/' || path[len]=='\0')) && 
+				if ((strncmp(path,fusetab[i]->fuse->path,len) == 0 && (path[len] == '/' || path[len]=='\0')) && !(isexception(path+len,fusetab[i]->fuse->exceptions,fusetab[i])) &&
 						((e=tst_matchingepoch(&(fusetab[i]->fuse->tst))) > maxepoch)) {
 					maxi=i;
 					maxepoch=e;
@@ -549,7 +565,7 @@ static void *startmain(void *vsmo)
 	/* handle -o options and specific filesystem options */
 	opts = mountflag2options(*(psmo->pmountflags), psmo->data);
 	fuse_set_context(psmo->new);
-	newargc=fuseargs(psmo->new->fuse->filesystemtype,psmo->source, psmo->new->fuse->path,opts, &newargv,psmo->new, &(psmo->new->fuse->flags));
+	newargc=fuseargs(psmo->new->fuse->filesystemtype,psmo->source, psmo->new->fuse->path,opts, &newargv,psmo->new, &(psmo->new->fuse->flags), &(psmo->new->fuse->exceptions));
 	free(opts);
 	if (psmo->new->fuse->flags & FUSE_DEBUG) {
 		GMESSAGE("UmFUSE Debug enabled");
@@ -566,6 +582,7 @@ static void *startmain(void *vsmo)
 	if ((newnewargv=malloc(newargc * sizeof (char *))) != NULL) {
 		for (i=0;i<newargc;i++) 
 			newnewargv[i]=newargv[i];
+		optind=1;
 		if (pmain(newargc,newnewargv) != 0)
 			umfuse_abort(psmo->new->fuse);
 		free(newnewargv);
@@ -809,6 +826,7 @@ static long umfuse_mount(char *source, char *target, char *filesystemtype,
 		new->fuse = (struct fuse *)malloc(sizeof(struct fuse));
 		assert(new->fuse);
 		new->fuse->path = strdup(target);
+		new->fuse->exceptions = NULL;
 		if (strcmp(target,"/")==0)
 			new->fuse->pathlen = 0;
 		else
@@ -835,7 +853,6 @@ static long umfuse_mount(char *source, char *target, char *filesystemtype,
 		smo.source = source;
 		smo.data = data;
 		
-		
 		pthread_cond_init(&(new->fuse->startloop),NULL);
 		pthread_cond_init(&(new->fuse->endloop),NULL);
 		pthread_mutex_init(&(new->fuse->endmutex),NULL);
@@ -855,6 +872,7 @@ static long umfuse_mount(char *source, char *target, char *filesystemtype,
 			pthread_join(fc_norace->fuse->thread, NULL);
 			dlclose(fc_norace->fuse->dlhandle);
 			free(fc_norace->fuse->filesystemtype);
+			freeexceptions(fc_norace->fuse->exceptions);
 			free(fc_norace->fuse->path);
 			free(fc_norace->fuse);
 			errno = EIO;
@@ -907,6 +925,7 @@ static long umfuse_umount2(char *target, int flags)
 		pthread_join(fc_norace->fuse->thread, NULL);
 		//printf("JOIN done\n");
 		free(fc_norace->fuse->filesystemtype);
+		freeexceptions(fc_norace->fuse->exceptions);
 		free(fc_norace->fuse->path);
 		dlclose(fc_norace->fuse->dlhandle);
 		free(fc_norace->fuse);
@@ -973,6 +992,57 @@ static int umfusefillreaddir(void *buf, const char *name, const struct stat *stb
 	return 0;
 }
 
+static int merge_newentry(char *name,struct umdirent *tail,struct umdirent *oldtail)
+{
+	if (oldtail) {
+		do {
+			struct umdirent *next=tail->next;
+			if (strcmp(next->de.d_name,name)==0)
+				return 0;
+			tail=next;
+		} while (tail != oldtail);
+	}
+	return 1;
+}
+
+static void um_mergedir(char *path,struct fuse_context *fc,fuse_dirh_t h)
+{
+	char *abspath;
+	int fd;
+	asprintf(&abspath,"%s%s",fc->fuse->path,path);
+	fd=open(abspath,O_RDONLY|O_DIRECTORY);
+	free(abspath);
+	if (fd) {
+		char buf[4096];
+		int len;
+		struct umdirent *oldtail=h->tail;
+		while ((len=getdents64(fd,buf,4096)) > 0) {
+			off_t off=0;
+			while (off<len) {
+				struct dirent64 *de=(struct dirent *)(buf+off);
+				if (merge_newentry(de->d_name,h->tail,oldtail))
+				{
+					struct umdirent *new=(struct umdirent *)malloc(sizeof(struct umdirent));
+					new->de.d_name=strdup(de->d_name);
+					new->de.d_type=de->d_type;
+					new->de.d_ino=de->d_ino;
+					new->de.d_reclen=WORDALIGN(SIZEDIRENT64NONAME+strlen(de->d_name)+1);
+					new->de.d_off=h->offset=h->offset+WORDALIGN(12+strlen(de->d_name));
+					if (h->tail==NULL) {
+						new->next=new;
+					} else {
+						new->next=h->tail->next;
+						h->tail->next=new;
+					}
+					h->tail=new;
+				}
+				off+=de->d_reclen;
+			}
+		}
+		close(fd);
+	}
+}
+
 static struct umdirent *umfilldirinfo(struct fileinfo *fi)
 {
 	int rv;
@@ -986,6 +1056,9 @@ static struct umdirent *umfilldirinfo(struct fileinfo *fi)
 	else
 		rv=fc->fuse->fops.getdir(fi->path, &dh, umfusefilldir);
 	fuse_set_context(oldfc);
+	
+	if (fc->fuse->flags & FUSE_MERGE && rv>=0) 
+		um_mergedir(fi->path,fc,&dh);
 	if (rv < 0)
 		return NULL;
 	else 
@@ -999,6 +1072,7 @@ static void umcleandirinfo(struct umdirent *tail)
 			struct umdirent *tmp;
 			tmp=tail->next;
 			tail->next=tmp->next;
+			free(tmp->de.d_name);
 			free(tmp);
 		}
 		free(tail);
@@ -1374,10 +1448,11 @@ static long umfuse_read(int fd, void *buf, size_t count)
 	}
 }
 
+static long umfuse_lseek(int fd, int offset, int whence);
 static long umfuse_write(int fd, void *buf, size_t count)
 {
 //TODO write page?!
-	int rv;
+	int rv=0;
 
 	if ( (filetab[fd]==NULL) || ((filetab[fd]->ffi.flags & O_ACCMODE) == O_RDONLY)) {
 		errno = EBADF;
@@ -1391,8 +1466,12 @@ static long umfuse_write(int fd, void *buf, size_t count)
 		struct fuse_context *fc=filetab[fd]->context;
 		struct fuse_context *oldfc=fuse_gs_context(fc);
 		fc->pid=um_mod_getpid();
-		rv = fc->fuse->fops.write(filetab[fd]->path,
-				buf, count, filetab[fd]->pos, &filetab[fd]->ffi);
+		if (filetab[fd]->ffi.flags & O_APPEND)
+			rv=umfuse_lseek(fd,0,SEEK_END);
+		if (rv!=-1) {
+			rv = fc->fuse->fops.write(filetab[fd]->path,
+					buf, count, filetab[fd]->pos, &filetab[fd]->ffi);
+		}
 		if (fc->fuse->flags & FUSE_DEBUG) {
 			GMESSAGE("WRITE[%s:%d] => path:%s count:0x%x rv:%d\n",
 				fc->fuse->path, fd, filetab[fd]->path, count, rv);
@@ -1435,6 +1514,16 @@ static int stat2stat64(struct stat64 *s64, struct stat *s)
 	return 0;
 }
 
+/*heuristics for file system which does not set st_ino */
+static inline unsigned long hashnodeid (const char *s) {
+	unsigned long sum=0;
+	while (*s) {
+		sum=sum ^ ((sum << 5) + (sum >> 2) + *s);
+		s++;
+	}
+	return sum;
+}
+
 static int common_stat(struct fuse_context *fc, char *path,  struct stat *buf,int wrapped)
 {
 	int rv;
@@ -1450,6 +1539,12 @@ static int common_stat(struct fuse_context *fc, char *path,  struct stat *buf,in
 				fc->fuse->path, path, rv ? "Error" : "Success", (rv < 0) ? -rv : 0);
 	}
 	fuse_set_context(oldfc);
+	/*heuristics for file system which does not set st_ino */
+	if (buf->st_ino == 0)
+		buf->st_ino=(ino_t) hashnodeid(path);
+	/*heuristics for file system which does not set st_dev */
+	if (buf->st_dev == 0)
+		buf->st_dev=(dev_t) fc;
 	if (rv<0) {
 		errno= -rv;
 		return -1;
@@ -1466,6 +1561,8 @@ static int common_stat64(struct fuse_context *fc, char *path,  struct stat64 *bu
 	return rv;
 }
 
+#if 0
+/* 09 2008 f-syscall unification */
 static long umfuse_fstat(int fd, struct stat *buf)
 {
 	if (fd < 0 || filetab[fd] == NULL) {
@@ -1509,6 +1606,7 @@ static long umfuse_fstat64(int fd, struct stat64 *buf64)
 		stat2stat64(buf64,&buf);
 	return rv;
 }
+#endif
 
 #if 0
 static long umfuse_stat(char *path, struct stat *buf)
@@ -2271,6 +2369,8 @@ static long umfuse_statfs64 (char *file, struct statfs64 *buf)
 	return 0;
 }
 
+#if 0
+/* 09 2008 f-syscall unification */
 static long umfuse_fstatfs64 (unsigned int fd, struct statfs64 *buf)
 {
 	if (filetab[fd]==NULL) {
@@ -2284,6 +2384,7 @@ static long umfuse_fstatfs64 (unsigned int fd, struct statfs64 *buf)
 		return rv;
 	}
 }
+#endif
 
 void contextclose(struct fuse_context *fc)
 {
@@ -2315,15 +2416,15 @@ init (void)
 #if __WORDSIZE == 32 //TODO: verify that ppc64 doesn't have these
 	SERVICESYSCALL(s, stat64, umfuse_stat64);
 	SERVICESYSCALL(s, lstat64, umfuse_lstat64);
-	SERVICESYSCALL(s, fstat64, umfuse_fstat64);
+	//SERVICESYSCALL(s, fstat64, umfuse_fstat64);
 	SERVICESYSCALL(s, statfs64, umfuse_statfs64);
-	SERVICESYSCALL(s, fstatfs64, umfuse_fstatfs64);
+	//SERVICESYSCALL(s, fstatfs64, umfuse_fstatfs64);
 #else 
 	SERVICESYSCALL(s, stat, umfuse_stat64);
 	SERVICESYSCALL(s, lstat, umfuse_lstat64);
-	SERVICESYSCALL(s, fstat, umfuse_fstat64);
+	//SERVICESYSCALL(s, fstat, umfuse_fstat64);
 	SERVICESYSCALL(s, statfs, umfuse_statfs64);
-	SERVICESYSCALL(s, fstatfs, umfuse_fstatfs64);
+	//SERVICESYSCALL(s, fstatfs, umfuse_fstatfs64);
 #endif
 	SERVICESYSCALL(s, readlink, umfuse_readlink);
 	SERVICESYSCALL(s, getdents64, umfuse_getdents64);
@@ -2358,7 +2459,7 @@ init (void)
 #endif
 	SERVICESYSCALL(s, pread64, umfuse_pread64);
 	SERVICESYSCALL(s, pwrite64, umfuse_pwrite64);
-	SERVICESYSCALL(s, utime, umfuse_utime);
+	//SERVICESYSCALL(s, utime, umfuse_utime);
 	SERVICESYSCALL(s, utimes, umfuse_utimes);
 	add_service(&s);
 }
