@@ -115,30 +115,6 @@ static void misc_set_context(struct misc *mc)
 }
 #endif
 
-static void cutdots(char *path)
-{
-	int l=strlen(path);
-	l--;
-	if (path[l]=='.') {
-		l--;
-		if(path[l]=='/') {
-			if (l!=0) path[l]=0; else path[l+1]=0;
-		} else if (path[l]=='.') {
-			l--;
-			if(path[l]=='/') {
-				while(l>0) {
-					l--;
-					if (path[l]=='/')
-						break;
-				}
-				if(path[l]=='/') {
-					if (l!=0) path[l]=0; else path[l+1]=0;
-				}
-			}
-		}
-	}
-}
-
 static struct ummisc *searchmisc(char *path,int exact)
 {
 	register int i;
@@ -149,7 +125,6 @@ static struct ummisc *searchmisc(char *path,int exact)
 
 	PRINTDEBUG(0,"SearchContext:%s\n",path);
 	//fprint2("SearchContext:%s\n",path);
-	cutdots(path);
 	for (i=0;i<misctabmax;i++)
 	{
 		epoch_t e;
@@ -330,8 +305,8 @@ static char *unwrap(struct ummisc *mh,char *path)
 static int strpathcmp(char *s1,char *s2)
 {
 	int c=0;
-	while (*s1 && *s2 && c==0)
-		c=s2-s1;
+	for (c=0;(c=*s2-*s1)==0 && *s1 && *s2;s1++,s2++)
+		;
 	if (*s1 == *s2 || *s1 == '/' || *s2 == '/')
 		return 0;
 	else
@@ -349,6 +324,7 @@ static struct fsentry *recsearch(struct fsentry *fsdir,struct fsentry *fse,char
 		if (fse==NULL || fse->name==NULL)
 			return NULL;
 		else {
+			//fprint2("|%s|%s| %d\n",fse->name,path,strpathcmp(fse->name,path));
 			if (strpathcmp(fse->name,path) == 0)
 			{
 				path+=strlen(fse->name);
@@ -364,13 +340,17 @@ static struct fsentry *searchentry(struct ummisc *mh,char *path)
 	return recsearch(&(mh->ummisc_ops->root),mh->ummisc_ops->root.subdir,path);
 }
 
+static struct fsentry nullroot[] = {
+	  {NULL,NULL,NULL,0}};
+
 static long ummisc_open(char *path, int flags, mode_t mode)
 {
 	struct ummisc *mh = searchmisc(path,SUBSTR);
 	assert(mh);
 	char *upath=unwrap(mh,path);
-	//fprint2("open %s\n",upath);
+	//fprint2("open |%s| %d\n",upath,*upath);
 	struct fsentry *fse=searchentry(mh,upath);
+	//fprint2("ummisc_open %s %p\n",upath,fse);
 	if (fse != NULL) {
 		int fi = addfiletab();
 		int rv;
@@ -381,7 +361,10 @@ static long ummisc_open(char *path, int flags, mode_t mode)
 		//fprint2("%d %lld %s\n",fi,filetab[fi]->pos,filetab[fi]->path);
 		filetab[fi]->fse=fse;
 		filetab[fi]->ummisc=mh;
-		if (fse->subdir != NULL) {
+		/* is a dir, root is always a dir! */
+		if (fse->subdir != NULL || *upath==0) {
+			if (fse->subdir==NULL)
+				fse->subdir=nullroot;
 			filetab[fi]->buf = NULL;
 			filetab[fi]->size = 0;
 		} else {
@@ -390,6 +373,7 @@ static long ummisc_open(char *path, int flags, mode_t mode)
 			filetab[fi]->size=fse->getputfun(UMMISC_GET,filetab[fi]->buf,MISCFILESIZE,mh,
 					filetab[fi]->fse->tag,filetab[fi]->path);
 			if (flags & O_TRUNC) filetab[fi]->size=0;
+			if (flags & O_APPEND) filetab[fi]->pos=filetab[fi]->size;
 		}
 		return fi;
 	} else {
@@ -477,6 +461,8 @@ static setstat64(struct stat64 *buf64, int isdir)
 	buf64->st_size=MISCFILESIZE;
 }
 
+#if 0
+/* 09 2008 f-syscall unification */
 static long ummisc_fstat64(int fd, struct stat64 *buf64)
 {
 	if (fd < 0 || filetab[fd] == NULL) {
@@ -487,6 +473,7 @@ static long ummisc_fstat64(int fd, struct stat64 *buf64)
 		return 0;
 	}
 }
+#endif
 
 static long ummisc_stat64(char *path, struct stat64 *buf64)
 {
@@ -662,6 +649,7 @@ static long ummisc_umount2(char *target, int flags)
 		delmisctab(mh);
 		free(mh->path);
 		free(mh);
+		return 0;
 	}
 }
 
@@ -701,7 +689,7 @@ init (void)
 	SERVICESYSCALL(s, close, ummisc_close);
 	SERVICESYSCALL(s, stat64, ummisc_stat64);
 	SERVICESYSCALL(s, lstat64, ummisc_stat64);
-	SERVICESYSCALL(s, fstat64, ummisc_fstat64);
+	//SERVICESYSCALL(s, fstat64, ummisc_fstat64);
 	SERVICESYSCALL(s, fcntl64, ummisc_fcntl64);
 	SERVICESYSCALL(s, fsync, ummisc_fsync);
 	SERVICESYSCALL(s, access, ummisc_access);
