@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: capture_nested.c 559 2008-06-13 11:32:55Z garden $
+ *   $Id: capture_nested.c 630 2008-09-21 08:21:11Z rd235 $
  *
  */   
 #include <stdio.h>
@@ -115,6 +115,9 @@ static char *nest_abspath(int dirfd, long laddr,struct npcb *npc,struct stat64 *
 	if (npc->erno)
 		return um_patherror;  //error
 	else
+#if 0
+		return strdup(um_cutdots(newpath));
+#endif
 		return strdup(newpath);
 }
 
@@ -215,7 +218,6 @@ service_t nchoice_unlinkat(int sc_number,struct npcb *npc) {
 		return service_check(CHECKPATH,npc->path,1);
 }
 
-/* choice function for nested calls: dirfd,link/path (1st,2nd arg,choice on 4th) */
 /* choice function for nested calls: dirfd,link/path (1st,2nd arg,choice on 4th) */
 service_t nchoice_pl4at(int sc_number,struct npcb *npc) {
 	npc->path=nest_abspath(npc->sysargs[0],npc->sysargs[1],npc,&(npc->pathstat),
@@ -371,14 +373,14 @@ int nw_sysclose(int scno,struct npcb *npc,service_t sercode,sysfun um_syscall)
 {
 	int rv;
 	int fd=npc->sysargs[0];
-	int lfd=fd2lfd(&umview_file,npc->sysargs[0]);
+	int lfd=fd2lfd(&umview_file,fd);
 	if (lfd >= 0 && lfd_getcount(lfd) <= 1) { //no more opened lfd on this file:
 		npc->sysargs[0]=fd2sfd(&umview_file,fd);
-		r_close(fd);
 		rv=do_nested_call(um_syscall,&(npc->sysargs[0]),scmap[uscno(scno)].nargs);
 		if (rv >= 0) {
 			lfd_nullsfd(lfd);
 			lfd_deregister_n_close(&umview_file,fd);
+			r_close(fd);
 		}
 		return rv;
 	} else
@@ -552,7 +554,7 @@ static int nested_sysindex(struct npcb *npc, int scno)
 /* dcif (index) for sockets or virtual: just the sysno*/
 static int nested_sockvirindex(struct npcb *npc, int scno)
 {
-	  return scno;
+	return scno;
 }
 
 
@@ -676,6 +678,7 @@ static long int capture_nested_virsc(long int sysno, ...){
 	/* commonwrap for nested socket calls, 
 	 * nested_commonwrap sets errno, so the following code should not
 	 * call any system call or errno must be saved*/
+	callee_pcb.private_scno=sysno | ESCNO_VIRSC;
 	rv=nested_commonwrap(sysno, &callee_pcb, nested_sockvirindex, nested_call_virsc, service_virsyscall, virscmap);
 
 	nrestoreargs(caller_pcb, &callee_pcb);
@@ -720,6 +723,7 @@ static long int capture_nested_socketcall(long int sysno, ...){
 	 * callee_pcb.umpid=caller_pcb->umpid;
 	 */
 	/* commonwrap for nested socket calls */
+	callee_pcb.private_scno=sysno | ESCNO_SOCKET;
 	rv=nested_commonwrap(sysno, &callee_pcb, nested_sockvirindex, nested_call_sockcall, service_socketcall, sockmap);
 
 	nrestoreargs(caller_pcb, &callee_pcb);
@@ -772,6 +776,7 @@ static long int capture_nested_syscall(long int sysno, ...)
    * callee_pcb.umpid=caller_pcb->umpid;
 	 */
 	/* commonwrap for nested calls */
+	callee_pcb.private_scno = sysno;
 	rv=nested_commonwrap(sysno, &callee_pcb, nested_sysindex, nested_call_syscall, service_syscall, scmap);
 
 	nrestoreargs(caller_pcb, &callee_pcb);
