@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: scmap.c 559 2008-06-13 11:32:55Z garden $
+ *   $Id: scmap.c 655 2009-04-28 18:37:08Z rd235 $
  *
  */   
 
@@ -68,6 +68,7 @@ wrapinfun wrap_in_statfs64, wrap_in_fstatfs64;
 wrapinfun wrap_in_mmap,wrap_in_mremap,wrap_in_munmap;
 #endif
 wrapinfun wrap_in_kill;
+wrapinfun wrap_in_getxid16,wrap_in_setuid16, wrap_in_setgid16;
 wrapinfun wrap_in_getxid,wrap_in_setuid,wrap_in_setreuid;
 wrapinfun wrap_in_getresuid, wrap_in_setresuid,wrap_in_setgid;
 wrapinfun wrap_in_setresgid,wrap_in_setregid;
@@ -115,6 +116,9 @@ wrapinfun wrap_in_uname, wrap_in_gethostname, wrap_in_sethostname;
 	#define	__NR_connect	SYS_CONNECT
 	#define	__NR_listen	SYS_LISTEN
 	#define	__NR_accept	SYS_ACCEPT
+#ifdef SYS_ACCEPT4
+	#define	__NR_accept4	SYS_ACCEPT4
+#endif
 	#define	__NR_getsockname	SYS_GETSOCKNAME 
 	#define	__NR_getpeername	SYS_GETPEERNAME
 	#define	__NR_socketpair	SYS_SOCKETPAIR
@@ -173,6 +177,9 @@ struct sc_map scmap[]={
 	{__NR_chroot,	choice_path,	wrap_in_chroot, wrap_out_chroot,	always_umnone,	NULL, ALWAYS,	1, SOC_FILE|SOC_NET},
 	{__NR_dup,	choice_fd,	wrap_in_dup,	wrap_out_dup,	nchoice_fd, nw_sysdup, ALWAYS,	1, SOC_FILE|SOC_NET},
 	{__NR_dup2,	choice_fd,	wrap_in_dup,	wrap_out_dup,	nchoice_fd, nw_sysdup, ALWAYS,	2, SOC_FILE|SOC_NET},
+#ifdef __NR_dup3
+	{__NR_dup3,	choice_fd,	wrap_in_dup,	wrap_out_dup,	nchoice_fd, nw_sysdup, ALWAYS,	3, SOC_FILE|SOC_NET},
+#endif
 	{__NR_mount,	choice_mount,	wrap_in_mount,	wrap_out_std,	always_umnone,	NULL, 0,	5, SOC_FILE},
 	{__NR_umount,	choice_path,	wrap_in_umount,	wrap_out_std,	always_umnone,	NULL, 0,	1, SOC_FILE},
 	{__NR_umount2,	choice_path,	wrap_in_umount2,wrap_out_std,	always_umnone,	NULL, 0,	2, SOC_FILE},
@@ -254,6 +261,9 @@ struct sc_map scmap[]={
 	{__NR_readlinkat, choice_linkat, wrap_in_readlink, wrap_out_std, nchoice_linkat, nw_sysatpath_std, 0, 4, SOC_FILE},
 	{__NR_fchmodat, choice_pl4at, wrap_in_chmod, wrap_out_std, nchoice_pl4at, nw_sysatpath_std, 0, 4, SOC_FILE},
 	{__NR_faccessat, choice_pl4at, wrap_in_access, wrap_out_std, nchoice_pl4at, nw_sysatpath_std, 0, 4, SOC_FILE},
+#ifdef __NR_utimensat
+	{__NR_utimensat, choice_pl4at, wrap_in_utime,  wrap_out_std, nchoice_pl4at, nw_sysatpath_std, 0, 4, SOC_FILE},
+#endif
 #endif
 
 #ifdef _UM_MMAP
@@ -283,20 +293,34 @@ struct sc_map scmap[]={
 	{__NR_setdomainname, choice_sc,	wrap_in_sethostname,  wrap_out_std,	always_umnone,	NULL, 0,	2, SOC_HOSTID},
 
 	/* user mgmt calls */
-	{__NR_getuid,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
-	{__NR_setuid,	choice_sc,	wrap_in_setuid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
-	{__NR_geteuid,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_getuid,	choice_sc,	wrap_in_getxid16, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setuid,	choice_sc,	wrap_in_setuid16, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_geteuid,	choice_sc,	wrap_in_getxid16, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
 	{__NR_setfsuid,	choice_sc,	wrap_in_setuid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
 	{__NR_setreuid,	choice_sc,	wrap_in_setreuid, wrap_out_std, 	always_umnone,	NULL, 0,	2, SOC_UID},
 	{__NR_getresuid, choice_sc,	wrap_in_getresuid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
 	{__NR_setresuid, choice_sc,	wrap_in_setresuid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
-	{__NR_getgid,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
-	{__NR_setgid,	choice_sc,	wrap_in_setgid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
-	{__NR_getegid,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
-	{__NR_setfsgid,	choice_sc,	wrap_in_setgid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_getgid,	choice_sc,	wrap_in_getxid16, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setgid,	choice_sc,	wrap_in_setgid16, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_getegid,	choice_sc,	wrap_in_getxid16, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setfsgid,	choice_sc,	wrap_in_setgid16, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
 	{__NR_setregid,	choice_sc,	wrap_in_setregid, wrap_out_std, 	always_umnone,	NULL, 0,	2, SOC_UID},
 	{__NR_getresgid, choice_sc,	wrap_in_setresuid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
 	{__NR_setresgid, choice_sc,	wrap_in_setresgid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
+	{__NR_getuid32,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setuid32,	choice_sc,	wrap_in_setuid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_geteuid32,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setfsuid32,	choice_sc,	wrap_in_setuid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setreuid32,	choice_sc,	wrap_in_setreuid, wrap_out_std, 	always_umnone,	NULL, 0,	2, SOC_UID},
+	{__NR_getresuid32, choice_sc,	wrap_in_getresuid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
+	{__NR_setresuid32, choice_sc,	wrap_in_setresuid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
+	{__NR_getgid32,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setgid32,	choice_sc,	wrap_in_setgid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_getegid32,	choice_sc,	wrap_in_getxid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setfsgid32,	choice_sc,	wrap_in_setgid, wrap_out_std, 	always_umnone,	NULL, 0,	1, SOC_UID},
+	{__NR_setregid32,	choice_sc,	wrap_in_setregid, wrap_out_std, 	always_umnone,	NULL, 0,	2, SOC_UID},
+	{__NR_getresgid32, choice_sc,	wrap_in_setresuid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
+	{__NR_setresgid32, choice_sc,	wrap_in_setresgid, wrap_out_std, 	always_umnone,	NULL, 0,	3, SOC_UID},
 	  
 	/* priority related calls */
 	{__NR_nice,	choice_sc,	wrap_in_nice,  wrap_out_std,	always_umnone,	NULL, 0,	1, SOC_PRIO},
@@ -347,6 +371,9 @@ struct sc_map sockmap[]={
 /*15*/	{__NR_getsockopt,choice_fd,	wrap_in_getsockopt,	wrap_out_std,	nchoice_sfd,	nw_sockfd_std, 0,	5, SOC_SOCKET|SOC_NET},
 /*16*/	{__NR_sendmsg,   choice_fd,	wrap_in_sendmsg,	wrap_out_std,	nchoice_sfd,	nw_sockfd_std, 0,	3, SOC_SOCKET|SOC_NET},
 /*17*/	{__NR_recvmsg,   choice_fd,	wrap_in_recvmsg,	wrap_out_std,	nchoice_sfd,	nw_sockfd_std, CB_R,	3, SOC_SOCKET|SOC_NET},
+#ifdef __NR_accept4
+/*18*/	{__NR_accept4,   choice_fd,	wrap_in_accept,	wrap_out_socket,	nchoice_sfd,	nw_accept, CB_R,	4, SOC_SOCKET|SOC_NET},
+#endif
 };
 
 /* fake sockmap when socket system calls are normal syscalls */
