@@ -19,7 +19,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: sctab.c 472 2008-04-28 10:06:51Z rd235 $
+ *   $Id: sctab.c 669 2009-05-30 16:24:08Z rd235 $
  *
  */   
 #include <assert.h>
@@ -148,12 +148,12 @@ int um_x_lstat64(char *filename, struct stat64 *buf, struct pcb *pc)
 	/* fprint2("-> um_lstat: %s\n",filename); */
 	/* internal nested call save data */
 	oldscno = pc->sysscno;
-	pc->sysscno = NR64_stat;
+	pc->sysscno = NR64_lstat;
 	epoch=pc->tst.epoch;
 	if ((sercode=service_check(CHECKPATH,filename,1)) == UM_NONE)
 		retval = r_lstat64(filename,buf);
 	else{
-		retval = service_syscall(sercode,uscno(NR64_stat))(filename,buf,pc);
+		retval = service_syscall(sercode,uscno(NR64_lstat))(filename,buf,pc);
 	}
 	/* internal nested call restore data */
 	pc->sysscno = oldscno;
@@ -196,6 +196,41 @@ char *um_getpath(long laddr,struct pcb *pc)
 		return um_patherror;
 }
 
+#if 0
+char *um_cutdots(char *path)
+{
+	int l=strlen(path);
+#ifdef CUTDOTSTEST
+	char *s=strdup(path);
+#endif
+	l--;
+	if (path[l]=='.') {
+		l--;
+		if(path[l]=='/') {
+			if (l!=0) path[l]=0; else path[l+1]=0;
+		} else if (path[l]=='.') {
+			l--;
+			if(path[l]=='/') {
+				while(l>0) {
+					l--;
+					if (path[l]=='/')
+						break;
+				}
+				if(path[l]=='/') {
+					if (l!=0) path[l]=0; else path[l+1]=0;
+				}
+			}
+		}
+	}
+#ifdef CUTDOTSTEST
+	if (strcmp(path,s) != 0)
+		fprint2("cutdots worked %s %s\n",path,s);
+	free(s);
+#endif
+	return path;
+}
+#endif
+
 /* get a path, convert it as an absolute path (and strdup it) 
  * from the process address space */
 char *um_abspath(int dirfd, long laddr,struct pcb *pc,struct stat64 *pst,int dontfollowlink)
@@ -213,6 +248,9 @@ char *um_abspath(int dirfd, long laddr,struct pcb *pc,struct stat64 *pst,int don
 		if (pc->erno)
 			return um_patherror;	//error
 		else
+#if 0
+			return strdup(um_cutdots(newpath));
+#endif
 			return strdup(newpath);
 	}
 	else {
@@ -648,7 +686,7 @@ service_t choice_mount(int sc_number,struct pcb *pc)
 		char filesystemtype[PATH_MAX];
 		unsigned long fstype=pc->sysargs[2];
 		if (umovestr(pc,fstype,PATH_MAX,filesystemtype) == 0) {
-			return service_check(CHECKFSTYPE,filesystemtype,0);
+			return service_check(CHECKFSTYPE,fs_alias(filesystemtype),0);
 		}
 		else
 			return UM_NONE;
@@ -660,7 +698,7 @@ service_t choice_mount(int sc_number,struct pcb *pc)
 service_t choice_path(int sc_number,struct pcb *pc)
 {
 	pc->path=um_abspath(AT_FDCWD,pc->sysargs[0],pc,&(pc->pathstat),0); 
-	//printf("choice_path %d %s\n",sc_number,pc->path);
+	//fprint2("choice_path %d %s\n",sc_number,pc->path);
 
 	if (pc->path==um_patherror){
 		/*		char buff[PATH_MAX];
