@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: services.c 542 2008-06-08 09:17:12Z rd235 $
+ *   $Id: services.c 647 2009-04-22 21:37:57Z rd235 $
  *
  */   
 #include <assert.h>
@@ -75,6 +75,8 @@ static struct syscall_unifier scunify[] = {
 	{__NR_readv,	__NR_read},
 	{__NR_writev,	__NR_write},
 	{__NR_time,	  __NR_gettimeofday},
+	{__NR_fchown,	__NR_chown},
+	{__NR_fchmod,	__NR_chmod},
 #if (__NR_olduname != __NR_doesnotexist)
 	{__NR_olduname, __NR_uname},
 #endif
@@ -89,18 +91,27 @@ static struct syscall_unifier scunify[] = {
 	{__NR_umount,	__NR_umount2},
 	{__NR_stat,		__NR_stat64},
 	{__NR_lstat,	__NR_lstat64},
-	{__NR_fstat,	__NR_fstat64},
+	{__NR_fstat,	__NR_stat64},
+	{__NR_fstat64,__NR_stat64},
 	{__NR_getdents,	__NR_getdents64},
 	{__NR_truncate,	__NR_truncate64},
 	{__NR_ftruncate,__NR_ftruncate64},
 	{__NR_statfs,	__NR_statfs64},
-	{__NR_fstatfs,	__NR_fstatfs64},
+	{__NR_fstatfs,	__NR_statfs64},
+	{__NR_fstatfs64,	__NR_statfs64},
+#else
+	{__NR_fstatfs,	__NR_statfs},
+	{__NR_fstat,	__NR_stat},
 #endif 
 	{__NR_openat,	__NR_open},
 	{__NR_mkdirat,	__NR_mkdir},
 	{__NR_mknodat,	__NR_mknod},
 	{__NR_fchownat,	__NR_chown},
 	{__NR_futimesat,	__NR_utimes},
+#ifdef __NR_utimensat
+	{__NR_utimensat,	__NR_utimes},
+#endif
+	{__NR_utime,	__NR_utimes},
 #ifdef __NR_newfstatat
 	{__NR_newfstatat,	__NR_stat64},
 #endif
@@ -114,6 +125,30 @@ static struct syscall_unifier scunify[] = {
 	{__NR_readlinkat,	__NR_readlink},
 	{__NR_fchmodat,	__NR_chmod},
 	{__NR_faccessat,	__NR_access},
+#if defined(__NR_getuid32) && __NR_getuid32 != __NR_getuid
+	{__NR_getuid, __NR_getuid32},
+	{__NR_getgid, __NR_getgid32},   
+	{__NR_geteuid, __NR_geteuid32},
+	{__NR_getegid, __NR_getegid32},
+	{__NR_setreuid, __NR_setreuid32},
+	{__NR_setregid, __NR_setregid32},
+	{__NR_getgroups, __NR_getgroups32},
+	{__NR_setgroups, __NR_setgroups32},
+	{__NR_setresuid, __NR_setresuid32},
+	{__NR_getresuid, __NR_getresuid32},
+	{__NR_setresgid, __NR_setresgid32},
+	{__NR_getresgid, __NR_getresgid32},
+	{__NR_setuid, __NR_setuid32},
+	{__NR_setgid, __NR_setgid32},
+	{__NR_setfsuid, __NR_setfsuid32},
+	{__NR_setfsgid, __NR_setfsgid32},
+#endif
+#if defined(__NR_chown32) && __NR_chown32 != __NR_chown
+	//{__NR_chown, __NR_chown32},
+	//{__NR_lchown, __NR_lchown32},
+	//{__NR_fchown, __NR_chown32},
+	{__NR_fchown32, __NR_chown32},
+#endif
 #ifdef SNDRCVMSGUNIFY
 #if (__NR_socketcall == __NR_doesnotexist)
 	{__NR_send, __NR_sendmsg},
