@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: um_basicio.c 472 2008-04-28 10:06:51Z rd235 $
+ *   $Id: um_basicio.c 649 2009-04-23 05:44:01Z rd235 $
  *
  */   
 #include <assert.h>
@@ -40,7 +40,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <linux/types.h>
-#include <linux/dirent.h>
+#include <linux_dirent.h>
 #include <config.h>
 #include "defs.h"
 #include "umproc.h"
@@ -89,7 +89,23 @@ int wrap_in_open(int sc_number,struct pcb *pc,
 		} else
 			return SC_FAKE;
 	} else {
+#if 0
+		/* load the complete path (to avoid border effects -- like open(..) */
+		/* open problem: all the paths should be rewritten on borders
+		 * not only for "open" */
+		int filenamelen=WORDALIGN(strlen(pc->path));
+		long sp=getsp(pc);
+		ustoren(pc,sp-filenamelen,filenamelen,pc->path);
+#ifdef __NR_openat
+		if (sc_number == __NR_openat) 
+			pc->sysargs[1]=sp-filenamelen;
+		else 
+#endif
+			pc->sysargs[0]=sp-filenamelen;
 		/* keep track of the open file */
+		pc->retval=lfd_open(sercode,-1,pc->path,flags,0);
+		return SC_CALLONXIT;
+#endif
 		pc->retval=lfd_open(sercode,-1,pc->path,flags,0);
 		return SC_TRACEONLY;
 	}
@@ -387,13 +403,13 @@ int wrap_in_fstat(int sc_number,struct pcb *pc,
 		                service_t sercode, sysfun um_syscall)
 {
 	long pbuf=pc->sysargs[1];
-	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
-	if (sfd < 0) {
+	char *path =fd_getpath(pc->fds,pc->sysargs[0]);
+	if (path == NULL) {
 		pc->retval= -1;
 		pc->erno= EBADF;
 	} else {
 		struct stat64 buf64;
-		if ((pc->retval = um_syscall(sfd,&buf64)) >= 0) {
+		if ((pc->retval = um_syscall(path,&buf64)) >= 0) {
 			struct kstat kbuf;
 			stat64_2kstat(&buf64,&kbuf);
 			ustoren(pc,pbuf,sizeof(struct kstat),(char *)&kbuf);
@@ -410,16 +426,18 @@ int wrap_in_stat64(int sc_number,struct pcb *pc,
 {
 	long pbuf;
 	struct stat64 buf;
+#if defined(__NR_fstatat64) || defined(__NR_newfstatat)
+	if (sc_number == 
 #ifdef __NR_fstatat64
-	if (sc_number == __NR_fstatat64)
-		pbuf=pc->sysargs[2];
-	else
+			__NR_fstatat64
 #else
-#ifdef __NR_newfstatat
-	if (sc_number == __NR_newfstatat)
-		pbuf=pc->sysargs[2];
-	else
+			__NR_newfstatat
 #endif
+		 ) {
+		pbuf=pc->sysargs[2];
+		if (pc->sysargs[3] & AT_SYMLINK_NOFOLLOW) 
+			um_syscall=service_syscall(sercode,uscno(NR64_lstat));
+	} else
 #endif
 	pbuf=pc->sysargs[1];
 	if ((pc->retval = um_syscall(pc->path,&buf)) >= 0)
@@ -433,14 +451,14 @@ int wrap_in_fstat64(int sc_number,struct pcb *pc,
 		                service_t sercode, sysfun um_syscall)
 {
 	long pbuf=pc->sysargs[1];
-	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
+	char *path=fd_getpath(pc->fds,pc->sysargs[0]);
 	/*fprint2("wrap_in_fstat: %d\n",sfd);*/
-	if (sfd < 0) {
+	if (path==NULL) {
 		pc->retval= -1;
 		pc->erno= EBADF;
 	} else {
 		struct stat64 buf;
-		if ((pc->retval = um_syscall(sfd,&buf)) >= 0)
+		if ((pc->retval = um_syscall(path,&buf)) >= 0)
 			ustoren(pc,pbuf,sizeof(struct stat64),&buf);
 		else
 			pc->erno=errno;
