@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: um_socket.c 559 2008-06-13 11:32:55Z garden $
+ *   $Id: um_socket.c 655 2009-04-28 18:37:08Z rd235 $
  *
  */   
 #include <assert.h>
@@ -211,9 +211,16 @@ int wrap_out_socket(int sc_number,struct pcb *pc) {
 	if (pc->behavior==SC_CALLONXIT && pc->retval >= 0) {
 		int fd=getrv(pc);	
 		/* if the syscall issued by the process was also okay */
-		if (fd >= 0) {
+		if (fd >= 0 && addfd(pc,fd) == 0) {
 			/* update open file table*/
 			lfd_register(pc->fds,fd,pc->retval);
+#ifdef __NR_accept4
+			if (sc_number == __NR_accept4) {
+				int flags = pc->sysargs[3];
+				if (flags & SOCK_CLOEXEC) 
+					fd_setfdfl(pc->fds,fd,FD_CLOEXEC);
+			}
+#endif
 		} else {
 			putrv(pc->retval,pc);
 			puterrno(pc->erno,pc);
